part of '../../ngx_core.dart';

/// Manages the user sessions.
class Session {

  /// The application instance.
  final Application _app;

  /// The cookie service.
  final Cookies _cookies;

  /// The identity object associated with the currently logged-in user.
  Identity _identity;

  /// The service providing access to the browser's window.
  final dom.Window _window;

  /// Creates a new session service.
  Session(@Optional() Cookies cookies, this._app, this._window):
    _cookies = cookies ?? Cookies(),
    endPoint = _app.apiUrl;

  /// The access token associated to this session.
  String get accessToken => _cookies['${_app.id}:access_token'] ?? '';
  set accessToken(String value) => _cookies['${_app.id}:access_token'] = value;

  /// The URL of the end point associated to this class.
  final Uri endPoint;

  /// The identity object associated with the currently logged-in user.
  Identity get identity => _identity;

  /// The configuration of the identity cookie.
  final CookieOptions identityCookie = CookieOptions();

  /// Value indicating whether the current user is a guest (not authenticated).
  bool get isGuest => identity == null;

  /// The URL of the login page.
  String loginUrl = '/login';

  /// The URL that the user should be redirected to after login.
  String get returnUrl => _window.sessionStorage['${_app.id}:returnUrl'] ?? '/';
  set returnUrl(String value) => _window.sessionStorage['${_app.id}:returnUrl'] = value;

  /// Logs out the current user and removes authentication-related cookie data.
  Future<void> logout() async => _destroy();

  /// Frees all session variables and destroys all data registered to a session.
  void _destroy() {
    _cookies..remove('${_app.id}:access_token')..remove('${_app.id}:identity');
    _identity = null;
  }
}
